# frozen_string_literal: true

module Gitlab::QA::Runtime
  RSpec.describe OmnibusConfiguration do
    subject { described_class.new }

    describe '#initialize' do
      it 'contains an initial comment denoting GitLab QA' do
        expect(subject.instance_variable_get(:@config).size).to eq(1)
        expect(subject.instance_variable_get(:@config).first).to include('GitLab QA')
      end

      context 'when prefix is specified' do
        let(:prefixed_configuration) { '# prefixed' }
        let(:generated_comment_configuration) do
          "# Generated by GitLab QA Omnibus Configurator at YYYY-MM-DD HH:MM:SS\n# prefixed"
        end

        it 'inserts configuration after the generation comment' do
          subject = described_class.new(prefixed_configuration)

          expect(subject.instance_variable_get(:@config).last).to eq(prefixed_configuration)
        end

        it 'removes any duplicated generated by comments' do
          subject = described_class.new(generated_comment_configuration)
          config = subject.instance_variable_get(:@config)

          expect(config.last.split("\n").size).to eq(1)
        end
      end
    end

    describe '#expand_config_template' do
      let(:gitlab) do
        instance_double(
          Gitlab::QA::Component::Gitlab,
          address: "http://gitlab.test",
          class: Struct.new(:name).new("Gitlab::QA::Component::Gitlab")
        )
      end

      it 'expands the ERB template' do
        subject.<< "gitlab_rails['a'] = external_url '<%= gitlab.address %>'"
        subject.expand_config_template(gitlab)
        expect(subject.instance_variable_get(:@config).last).to include('http://gitlab.test')
      end
    end

    describe '#sanitize!' do
      it 'removes duplicate omnibus entries' do
        subject.<< "gitlab_rails['a'] = true"
        subject.<< "gitlab_rails['a'] = true" # intentional duplicate
        subject.<< "gitlab_rails['b'] = true"

        expect(subject.sanitize!.size).to eq(3) # +1 from the configurator comment
      end

      it 'removes duplicate omnibus entries in multiline config' do
        subject.<< <<~OMNIBUS
          a['setting_1'] = true
          a['setting_3'] = false
        OMNIBUS

        subject.<< <<~OMNIBUS
          a['setting_1'] = true
          a['setting_4'] = false
        OMNIBUS

        expect(subject.sanitize!.size).to eq(4) # +1 from the configurator comment
      end

      it 'splits each setting into an array item' do
        subject.<< <<~OMNIBUS
          a['setting_1'] = true
          external_url 'http://gitlab.test
          a['setting_2'] = [
            {
              name: 'setting_2a_name',
              id: 'setting_2a_id'
            },
            {
              name: 'setting_2b_name',
              id: 'setting_2b_id'
            },
          ]
          a['setting_3'] = false
        OMNIBUS

        expect(subject.sanitize!.size).to eq(5) # +1 from the configurator comment
      end

      it 'merges entries with an array' do
        subject.<< <<~OMNIBUS
          gitlab_rails['omniauth_providers'] = [
            {
              name: 'github',
              app_id: '$QA_GITHUB_OAUTH_APP_ID',
              app_secret: '$QA_GITHUB_OAUTH_APP_SECRET',
              url: 'https://github.com/',
              verify_ssl: false,
              args: { scope: 'user:email' }
            }
          ]
          gitlab_rails['omniauth_allow_single_sign_on'] = ['github']
        OMNIBUS

        expected_output_1 = <<~OMNIBUS
          gitlab_rails['omniauth_providers'] = [
            {
              name: 'github',
              app_id: '$QA_GITHUB_OAUTH_APP_ID',
              app_secret: '$QA_GITHUB_OAUTH_APP_SECRET',
              url: 'https://github.com/',
              verify_ssl: false,
              args: { scope: 'user:email' }
            }]
        OMNIBUS

        expected_output_2 = "gitlab_rails['omniauth_allow_single_sign_on'] = ['github']"

        expect(subject.sanitize![1]).to eq expected_output_1.chomp
        expect(subject.sanitize![2]).to eq expected_output_2.chomp
      end

      it 'detects duplicate entries' do
        subject.<< "gitlab_rails['a'] = true"
        subject.<< "gitlab_rails['a'] = false"

        expect { subject.sanitize! }.to raise_error(RuntimeError)
      end

      it 'removes duplicate omnibus entries in multiline config' do
        subject.<< <<~OMNIBUS
          a['setting_1'] = false
          a['setting_3'] = false
        OMNIBUS

        subject.<< <<~OMNIBUS
          a['setting_1'] = true
          a['setting_4'] = false
        OMNIBUS

        expect { subject.sanitize! }.to raise_error(RuntimeError)
      end

      it 'sanitizes a malformed configuration' do
        subject.<< "gitlab_rails['a']=true"

        # well formed
        # gitlab_rails['a'] = true
        # vs gitlab_rails['a']=true
        expect(subject.sanitize!.last).to eq("gitlab_rails['a'] = true")
      end

      it 'allows comments' do
        subject.<< "# test"
        subject.<< "a = true"

        expect(subject.sanitize!.size).to eq(3) # +1 from the configurator comment
      end

      it 'leaves multiple equal signs alone' do
        subject.<< <<~RUBY
          a['test'] = { "key" => "value" }
        RUBY

        expect(subject.sanitize!.last).to eq(%(a['test'] = { 'key' => 'value' }))
      end

      it 'converts double quotes into single quotes' do
        subject.<< %(a['test'] = { "foo" => "bar" })

        expect(subject.sanitize!.last).to eq("a['test'] = { 'foo' => 'bar' }")
      end

      it 'allows space-separated DSL-type entries' do
        subject.<< %(mattermost_url 'abc123.test')
        expect(subject.sanitize!.last).to eq("mattermost_url 'abc123.test'")
      end

      it 'method invocations' do
        subject.<< <<~OMNIBUS
          git_data_dirs({
            'default' => '/test/default',
            'alternative' => '/test/alternative'
          })
        OMNIBUS

        expect(subject.sanitize!.size).to eq(2)
      end
    end

    describe '#<<' do
      it 'adds the config to the base configuration' do
        subject.<< "gitlab_rails['a'] = true"

        expect(subject.instance_variable_get(:@config).size).to eq(2) # +1 from the configurator comment
      end

      it 'strips any newlines off' do
        subject.<< "gitlab_rails['a'] = true\n"

        expect(subject.instance_variable_get(:@config).last).to eq("gitlab_rails['a'] = true")
      end

      it 'ignores empty configurations' do
        subject.<< "\n"

        expect(subject.instance_variable_get(:@config).size).to eq(1)
      end
    end

    describe OmnibusConfigurations::Default do
      describe '#configuration' do
        it 'has a default configuration' do
          expect(subject.configuration).not_to be_empty
        end

        it 'is a String type' do
          expect(subject.configuration).to be_a String
        end

        it 'sets a theme' do
          expect(subject.configuration).to include("gitlab_rails['gitlab_default_theme']")
        end

        it 'disables animations' do
          expect(subject.configuration).to include("gitlab_rails['gitlab_disable_animations'] = true")
        end

        it 'sets cache expiry' do
          expect(subject.configuration).to include("gitlab_rails['application_settings_cache_seconds'] = 0")
        end
      end
    end

    describe OmnibusConfigurations::ObjectStorage do
      it 'has a prepration method' do
        expect(described_class.instance_methods(false)).to include(:prepare)
      end

      it 'has exec commands' do
        expect(described_class.instance_methods(false)).to include(:exec_commands)
      end

      describe '#prepare' do
        subject { described_class.new }

        it 'returns an instance of Component::Base' do
          expect(subject.prepare).to be_a(Gitlab::QA::Component::Base)
        end
      end
    end
  end
end
